@@ -3,37 +3,39 @@ use Moose;
 
 extends 'Chart::Clicker::Renderer';
 
+use Graphics::Primitive::Brush;
+use Graphics::Primitive::Paint::Solid;
+use Graphics::Primitive::Operation::Fill;
+use Graphics::Primitive::Operation::Stroke;
+
 has '+additive' => ( default => 1 );
-has 'opacity' => (
-    is => 'rw',
-    isa => 'Num',
-    default => 0
-);
 has 'bar_padding' => (
     is => 'rw',
     isa => 'Int',
     default => 0
 );
-has 'stroke' => (
+has 'brush' => (
     is => 'rw',
-    isa => 'Graphics::Primitive::Stroke',
-    default => sub { Graphics::Primitive::Stroke->new() }
+    isa => 'Graphics::Primitive::Brush',
+    default => sub { Graphics::Primitive::Brush->new }
+);
+has 'opacity' => (
+    is => 'rw',
+    isa => 'Num',
+    default => 0
 );
 
-
 override('prepare', sub {
-    my $self = shift();
+    my ($self) = @_;
 
     super;
 
     my $dses = $self->clicker->get_datasets_for_context($self->context);
+    # my $raxis = $self->clicker->get_context($self->context)->range_axis;
 
-    # Find the largest number of keys.  Should we really do this?  We could
-    # probably just take the first since stacked bars doesn't work if the data
-    # doesn't match up.
     foreach my $ds (@{ $dses }) {
         if(!defined($self->{KEYCOUNT})) {
-            $self->{KEYCOUNT} = $ds->max_key_count();
+            $self->{KEYCOUNT} = $ds->max_key_count;
         }
         $self->{SCOUNT} += $ds->count;
     }
@@ -41,11 +43,10 @@ override('prepare', sub {
     return 1;
 });
 
-override('draw', sub {
-    my $self = shift();
+override('pack', sub {
+    my ($self) = @_;
 
     my $clicker = $self->clicker;
-    my $cr = $clicker->cairo;
 
     my $height = $self->height;
     my $width = $self->width;
@@ -55,12 +56,13 @@ override('draw', sub {
     my $domain = $ctx->domain_axis;
     my $range = $ctx->range_axis;
 
-    my $padding = $self->bar_padding();
+    my $padding = $self->bar_padding;
 
-    my $strokewidth = $self->stroke->width();
+    my $strokewidth = $self->brush->width;
     $padding += $strokewidth;
 
-    $self->{BWIDTH} = int($width / ($self->{KEYCOUNT}));
+    $self->{BWIDTH} = int(($width - ($width * $domain->fudge_amount)
+        - ($padding / 2 * $self->{KEYCOUNT})) / ($self->{KEYCOUNT}));
     $self->{HBWIDTH} = $self->{BWIDTH} / 2;
 
     # Fetch all the colors we'll need.  Since we build each vertical bar from
@@ -72,7 +74,7 @@ override('draw', sub {
     my @keys = @{ $dses->[0]->get_series(0)->keys };
 
     # Iterate over each key...
-    for (my $i = 0; $i < $self->{KEYCOUNT}; $i++) {
+    for (my $i = 0; $i < scalar(@keys); $i++) {
 
         # Get all the values from every dataset's series for each key
         my @values;
@@ -81,39 +83,43 @@ override('draw', sub {
         }
 
         # Mark the x, since it's the same for each Y value
-        my $x = $domain->mark($keys[$i]);
+        my $x = $domain->mark($width, $keys[$i],);
         my $accum = 0;
 
+        my $val = 0;
         for(my $j = 0; $j < scalar(@values); $j++) {
-            my $y = $range->mark($values[$j]);
+            $val += $values[$j];
+            my $y = $range->mark($height, $val);
 
-            $cr->rectangle(
-                $x - $self->{HBWIDTH}, $height - $y - $accum,
-                $self->{BWIDTH}, $y
-            );
+            $self->move_to($x - $self->{HBWIDTH}, $height - $y + $self->brush->width * 2);
+            $self->rectangle($self->{BWIDTH}, $y - $accum - 1);
             # Accumulate the Y value, as it dictates how much we bump up the
             # next bar.
-            $accum += $y;
+            $accum += $y - $accum;
 
             my $color = $self->{COLORS}->[$j];
 
-            my $fillcolor;
-            if($self->opacity()) {
-                $fillcolor = $color->clone();
-                $fillcolor->alpha($self->opacity());
+            my $fillop = Graphics::Primitive::Operation::Fill->new(
+                paint => Graphics::Primitive::Paint::Solid->new
+            );
+
+            if($self->opacity) {
+                my $fillcolor = $color->clone;
+                $fillcolor->alpha($self->opacity);
+                $fillop->paint->color($fillcolor);
+                # Since we're going to stroke this, we want to preserve it.
+                $fillop->preserve(1);
             } else {
-                $fillcolor = $color;
+                $fillop->paint->color($color);
             }
 
-            $cr->set_source_rgba($fillcolor->as_array_with_alpha());
-            $cr->fill_preserve();
+            $self->do($fillop);
 
-            $cr->set_line_width($strokewidth);
-            $cr->set_line_cap($self->stroke->line_cap());
-            $cr->set_line_join($self->stroke->line_join());
-
-            $cr->set_source_rgba($color->as_array_with_alpha());
-            $cr->stroke();
+            if($self->opacity) {
+                my $strokeop = Graphics::Primitive::Operation::Stroke->new;
+                $strokeop->brush->color($color);
+                $self->do($strokeop);
+            }
         }
     }
 
@@ -143,15 +149,15 @@ Chart::Clicker::Renderer::StackedBar renders a dataset as stacked bars.
 
 =over 4
 
-=item I<opacity>
-
-If true this value will be used when setting the opacity of the bar's fill.
-
-=item I<padding>
+=item I<bar_padding>
 
 How much padding to put around a bar.  A padding of 4 will result in 2 pixels
 on each side.
 
+=item I<opacity>
+
+If true this value will be used when setting the opacity of the bar's fill.
+
 =item I<stroke>
 
 A stroke to use on each bar.
